# Important colors (strong contrast, emphasis)
headers_colors_dict = {'part;': '#B8860B',
                       'item;': '#BDB76B',
                       'bullet point;': '#FAFAD2', # LightGoldenRodYellow
                       'table;': '#FAFAD2', # LightGoldenRodYellow
                       'ignore;': '#E0FFFF', # Light Cyan
                          'signature;': '#E0FFFF', # Light Cyan
                       }

# Less important colors (muted tones, background elements)
headers_colors_list = [
    "#f5f5f5",  # Very Light Gray
    "#e0e0e0",  # Light Gray
    "#d6d6d6",  # Light Gray
    "#cccccc",  # Light Gray
    "#d9d9d9",  # Very Light Gray
    "#f0f0f0",  # Even Lighter Gray
    "#e6e6e6",  # Very Light Gray
    "#f2f2f2",  # Even Lighter Gray

    # Muted colors with lower saturation
    "#d3d3d3",  # Muted Gray
    "#d0cccc",  # Muted Blue
    "#e0e0e6",  # Muted Lavender
    "#e6e0e0",  # Muted Pink
    "#d9e0e0",  # Muted Teal
    "#e6dfd6",  # Muted Green
    "#e0d6d6",  # Muted Peach
    "#dcdcdc",  # Muted Silver

    # Even more muted colors
    "#e5e5e5",  # Very Light Gray (more muted)
    "#cfcfcf",  # Light Gray (more muted)
    "#bdbdbd",  # Light Gray (more muted)
    "#bebebe",  # Light Gray (more muted)
    "#cecece",  # Very Light Gray (more muted)
    "#ebeb eb",  # Even Lighter Gray (more muted)
    "#cfcfcf",  # Very Light Gray (more muted)
    "#d7d7d7",  # Even Lighter Gray (more muted)

    # Muted colors with even lower saturation (carefully consider contrast)
    "#d9d9d9",  # Muted Gray (even lower saturation)
    "#c2c2c2",  # Muted Blue (even lower saturation)
    "#d6d6d9",  # Muted Lavender (even lower saturation)
    "#d9d3d3",  # Muted Pink (even lower saturation)
    "#d0d3d3",  # Muted Teal (even lower saturation)
    "#dfd9d9",  # Muted Green (even lower saturation)
    "#d3d6d6",  # Muted Peach (even lower saturation)
    "#cacaca",  # Muted Silver (even lower saturation)
]
